<?php

namespace Realestate\Sample\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Realestate\Sample\Http\Requests\InsertRequest;
use Realestate\Sample\Interfaces\InsertRepositoryInterface;
use Realestate\Sample\Models\Insert;

/**
 * Resource controller class for insert.
 */
class InsertResourceController extends BaseController
{

    /**
     * Initialize insert resource controller.
     *
     * @param type InsertRepositoryInterface $insert
     *
     * @return null
     */
    public function __construct(InsertRepositoryInterface $insert)
    {
        parent::__construct();
        $this->repository = $insert;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Realestate\Sample\Repositories\Criteria\InsertResourceCriteria::class);
    }

    /**
     * Display a list of insert.
     *
     * @return Response
     */
    public function index(InsertRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Realestate\Sample\Repositories\Presenter\InsertPresenter::class)
                ->$function();
        }

        $inserts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('sample::insert.names'))
            ->view('sample::insert.index', true)
            ->data(compact('inserts', 'view'))
            ->output();
    }

    /**
     * Display insert.
     *
     * @param Request $request
     * @param Model   $insert
     *
     * @return Response
     */
    public function show(InsertRequest $request, Insert $insert)
    {

        if ($insert->exists) {
            $view = 'sample::insert.show';
        } else {
            $view = 'sample::insert.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('sample::insert.name'))
            ->data(compact('insert'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new insert.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InsertRequest $request)
    {

        $insert = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('sample::insert.name')) 
            ->view('sample::insert.create', true) 
            ->data(compact('insert'))
            ->output();
    }

    /**
     * Create new insert.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InsertRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $insert                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('sample::insert.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('sample/insert/' . $insert->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/sample/insert'))
                ->redirect();
        }

    }

    /**
     * Show insert for editing.
     *
     * @param Request $request
     * @param Model   $insert
     *
     * @return Response
     */
    public function edit(InsertRequest $request, Insert $insert)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('sample::insert.name'))
            ->view('sample::insert.edit', true)
            ->data(compact('insert'))
            ->output();
    }

    /**
     * Update the insert.
     *
     * @param Request $request
     * @param Model   $insert
     *
     * @return Response
     */
    public function update(InsertRequest $request, Insert $insert)
    {
        try {
            $attributes = $request->all();

            $insert->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('sample::insert.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('sample/insert/' . $insert->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('sample/insert/' . $insert->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the insert.
     *
     * @param Model   $insert
     *
     * @return Response
     */
    public function destroy(InsertRequest $request, Insert $insert)
    {
        try {

            $insert->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('sample::insert.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('sample/insert/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('sample/insert/' . $insert->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple insert.
     *
     * @param Model   $insert
     *
     * @return Response
     */
    public function delete(InsertRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('sample::insert.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('sample/insert'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/sample/insert'))
                ->redirect();
        }

    }

    /**
     * Restore deleted inserts.
     *
     * @param Model   $insert
     *
     * @return Response
     */
    public function restore(InsertRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('sample::insert.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/sample/insert'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/sample/insert/'))
                ->redirect();
        }

    }

}
